import logging
import sys
import threading


logger = logging.getLogger("pyramid_skins")

PY2 = sys.version_info[0] == 2


def string(s):
    if isinstance(s, str):
        return s
    if PY2:
        return s.encode('utf-8')
    return s.decode('utf-8')


class Discoverer(threading.Thread):
    run = None

    def __init__(self):
        super(Discoverer, self).__init__()
        self.paths = {}

    def watch(self, path, handler):
        if self.run is None:  # pragma: nocover
            raise ImportError(
                "Must have either ``MacFSEvents`` (on Mac OS X) or "
                "``pyinotify`` (Linux) to enable runtime discovery.")

        self.paths[path] = handler

        # thread starts itself
        if not self.isAlive():
            self.daemon = True
            self.start()

    try:
        import fsevents
    except ImportError:  # pragma: nocover
        pass
    else:
        def run(self):  # pragma: nocover
            logger.info("Starting FS event listener.")

            def callback(subpath, subdir):
                for path, handler in self.paths.items():
                    path = string(path)
                    if subpath.startswith(path):
                        config = handler.configure()
                        config.commit()

            stream = self.fsevents.Stream(callback, *(string(x) for x in self.paths))
            observer = self.fsevents.Observer()
            observer.schedule(stream)
            observer.run()
            observer.unschedule(stream)
            observer.stop()
            observer.join()

    try:
        import pyinotify
    except ImportError:  # pragma: nocover
        pass
    else:
        wm = pyinotify.WatchManager()

        def run(self):  # pragma: nocover
            self.watches = []
            mask = self.pyinotify.IN_CREATE

            for path in self.paths:
                wdd = self.wm.add_watch(path, mask, rec=True)
                self.watches.append(wdd)

            class Event(self.pyinotify.ProcessEvent):
                def process_IN_CREATE(inst, event):
                    subpath = event.path
                    for path, handler in self.paths.items():
                        if subpath.startswith(path):
                            config = handler.configure()
                            config.commit()

            handler = Event()
            notifier = self.pyinotify.Notifier(self.wm, handler)
            notifier.loop()
            for wdd in self.watches:
                self.wm.rm_watch(wdd.values())
            notifier.stop()
            notifier.join()



import os
import unittest
import doctest

OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE |
               doctest.REPORT_ONLY_FIRST_FAILURE)

class DoctestCase(unittest.TestCase):
    def __new__(self, test):
        return getattr(self, test)()

    @classmethod
    def test_docs(cls):
        import manuel.testing
        import manuel.codeblock
        import manuel.doctest
        import manuel.capture
        m = manuel.doctest.Manuel(optionflags=OPTIONFLAGS)
        m += manuel.codeblock.Manuel()
        m += manuel.capture.Manuel()

        import pkg_resources
        return manuel.testing.TestSuite(
            m,
            pkg_resources.resource_filename(
                "pyramid_skins", os.path.join(
                    "docs", "registration.rst")),
            pkg_resources.resource_filename(
                "pyramid_skins", os.path.join(
                    "docs", "templates.rst")),
            pkg_resources.resource_filename(
                "pyramid_skins", os.path.join(
                    "docs", "routing.rst")),
            setUp=cls.setUp,
            tearDown=cls.tearDown,
            )

    @staticmethod
    def setUp(test):
        import pyramid.testing

        request = pyramid.testing.DummyRequest()
        config = pyramid.testing.setUp(request=request, hook_zca=True)

        config.add_route('/unused', 'unused')

        import pyramid_zcml
        pyramid_zcml.load_zcml(config, 'pyramid_zcml:meta.zcml')
        pyramid_zcml.load_zcml(config, 'pyramid_skins:meta.zcml')

        def xmlconfig(s, config=config):
            from zope.configuration.config import ConfigurationMachine
            from zope.configuration.xmlconfig import registerCommonDirectives
            from zope.configuration.xmlconfig import string

            context = ConfigurationMachine()
            context.config_class = type(config)
            context.autocommit = True
            context.registry = config.registry
            context.route_prefix = None
            context.actions = config.action_state.actions
            context.introspection = False

            registerCommonDirectives(context)

            string(s, context=context, execute=False)
            config.commit()

        test.globs['add_route'] = config.add_route
        test.globs['xmlconfig'] = xmlconfig
        test.globs['registry'] = config.registry
        test.globs['request'] = request

    @staticmethod
    def tearDown(test):
        test.globs.clear()
        import pyramid.testing
        pyramid.testing.tearDown()



from pyramid.traversal import ModelGraphTraverser

class RoutesTraverserFactory(ModelGraphTraverser):
    def __call__(self, environ):
        tdict = super(RoutesTraverserFactory, self).__call__(environ)
        tdict['view_name'] = '_'.join(tdict['subpath'])
        return tdict

def RoutesTraverser(factory):
    return factory



import os
import pkg_resources
import weakref

from pyramid.asset import resolve_asset_spec
from pyramid.compat import bytes_, text_
from pyramid.path import caller_path
from pyramid_skins.interfaces import ISkinObject
from pyramid_skins.interfaces import ISkinObjectFactory
from pyramid_skins.models import SkinObject


def walk(path):
    os.lstat(path)
    for dir_path, dirs, filenames in os.walk(path):
        for filename in filenames:
            full_path = os.path.join(dir_path, filename)
            rel_path = full_path[len(path) + 1:]
            yield rel_path.replace(os.path.sep, '/'), full_path


def dirs(path):
    os.lstat(path)
    for dir_path, dirs, filenames in os.walk(path):
        yield dir_path[len(path) + 1:]


class Adapter:
    def __init__(self, inst):
        self.inst = inst

    def __call__(self, request):
        return self.inst

    def refresh(self):
        return self.inst.refresh()


def register_skin_object(registry, relative_path, path, request_type):
    ext = os.path.splitext(path)[1]
    factory = registry.queryUtility(ISkinObjectFactory, name=ext) or SkinObject

    name = factory.component_name(relative_path)

    if request_type is not None:
        inst = registry.adapters.lookup(
            (request_type, ), ISkinObject, name=name, default=None)
    else:
        inst = registry.queryUtility(ISkinObject, name=name)

    if inst is not None:
        inst.path = path
        inst.refresh()
    else:
        inst = factory(relative_path, path)

        if request_type is not None:
            registry.registerAdapter(
                Adapter(inst), (request_type, ), ISkinObject, name=name)
        else:
            registry.registerUtility(inst, ISkinObject, name)


def register_skin_view(config, relative_path, path, kwargs):
    for inst in config.registry.getAllUtilitiesRegisteredFor(ISkinObject):
        if inst.path == path:
            break
    else:  # pragma: nocover
        raise RuntimeError("Skin object not found: %s." % relative_path)

    name = type(inst).component_name(relative_path).replace('/', '_')

    config.add_view(view=inst, name=name, **kwargs)


class Skins(object):
    threads = weakref.WeakValueDictionary()

    def __init__(self, config, path=None, discovery=False, request_type=None):
        self.config = config
        self.registry = config.registry
        self.path = os.path.realpath(text_(path))
        self.views = []
        self.request_type = config.maybe_dotted(request_type)

        if discovery:
            thread = self.threads.get(id(self.registry))
            if thread is None:
                from pyramid_skins.discovery import Discoverer
                thread = self.threads[id(self.registry)] = Discoverer()
            thread.watch(self.path, self)

    def __call__(self):
        registry = self.registry

        for relative_path, path in walk(self.path):
            yield (
                (relative_path, path, ISkinObject),
                register_skin_object,
                (registry, relative_path, path, self.request_type))

        objects = {}
        for relative_path, path in walk(self.path):
            objects[relative_path] = path

        for index, kwargs in self.views:
            if index is not None:
                for path in dirs(self.path):
                    relative_path = os.path.join(path, index)
                    skin_path = objects.get(relative_path)
                    if skin_path is not None:
                        objects[path] = skin_path

            for relative_path, path in objects.items():
                yield (
                    (
                        relative_path, path, ISkinObject,
                        kwargs.get('name'), kwargs.get('request_type'),
                        kwargs.get('route_name'), kwargs.get('request_method'),
                        kwargs.get('request_param'), kwargs.get('containment'),
                        kwargs.get('attr'), kwargs.get('renderer'),
                        kwargs.get('wrapper'), kwargs.get('xhr'),
                        kwargs.get('accept'), kwargs.get('header'),
                        kwargs.get('path_info')),
                    register_skin_view,
                    (self.config, relative_path, path, kwargs))

    def configure(self):
        for action in self():
            self.config.action(*action)

        return self.config

    def view(self, context, index=None, **kwargs):
        assert 'name' not in kwargs
        kwargs.setdefault('request_type', self.request_type)
        self.views.append((index, kwargs))


def register_path(config, spec, discovery=False, indexes=[], request_type=None):
    """ Add a skin path to the current configuration state.

    If ``discovery`` is enabled, the path will automatically be
    monitored for changes.

    The ``indexes`` argument is an optional list of view registrations
    with the provided names.

    The ``request_type`` option decides the request type for which to
    make the registration.
    """

    package_name, path = resolve_asset_spec(spec)
    if package_name is not None:
        path = pkg_resources.resource_filename(package_name, path)
    else:
        path = caller_path(path)

    directive = Skins(config, path, discovery, request_type)
    for index in indexes:
        directive.view(config, index)

    for action in directive():
        config.action(*action)

    return directive



from pyramid.interfaces import IRenderer
from zope.interface import implementer

from .interfaces import ISkinObject


@implementer(IRenderer)
def renderer_factory(info):
    """ Renders a skin object based on view name. """

    registry = info.registry

    def renderer(value, system):
        try:
            system.update(value)
        except (TypeError, ValueError):  # pragma: nocover
            raise ValueError('Renderer was passed non-dictionary as value.')

        try:
            name = system['view'].__name__
        except AttributeError:
            # handles the case of @view_config applied to instance methods
            name = system['request'].view_name

        request = system['request']

        instance = registry.queryAdapter(request, ISkinObject, name=name) or \
                   registry.getUtility(ISkinObject, name=name)

        return instance.render(**system)
    return renderer



from zope import interface
from zope.schema import TextLine
from zope.schema import Bool

from zope.configuration.fields import Path

from pyramid_zcml import IViewDirective
from pyramid_skins.configuration import Skins
from pyramid_skins.configuration import register_skin_object
from pyramid.config import Configurator


def skins(context, **kw):
    config = Configurator(registry=context.registry, autocommit=True)
    return Skins(config, **kw)


class ISkinDirective(interface.Interface):
    path = Path(
        title=u"Path",
        description=u"Path to the directory containing the skin.",
        required=True)

    discovery = Bool(
        title=u"Discovery",
        description=(u"Enables run-time discovery."),
        required=False)

    request_type = TextLine(
        title=u"The request type string or dotted name interface.",
        description=(u"If provided, skin objects will be registered "
                     u"as named adapters for the request type interface."),
        required=False)


class ISkinViewDirective(IViewDirective):
    index = TextLine(
        title=u"Index filename",
        description=u"""
        Filename for which index views are created.""",
        required=False)



import os
import unittest

path = os.path.dirname(__file__)


class ImperativeConfigurationTest(unittest.TestCase):
    def setUp(self):
        import pyramid.testing
        self.config = pyramid.testing.setUp(hook_zca=True, autocommit=True)

    def tearDown(self):
        import pyramid.testing
        pyramid.testing.tearDown()

    def test_register_absolute_path(self):
        from pyramid_skins.configuration import register_path
        register_path(self.config, os.path.join(path, 'skins'))

        from pyramid_skins.interfaces import ISkinObject
        inst = self.config.registry.queryUtility(ISkinObject, name="index.pt")
        self.assertTrue(inst is not None)

    def test_register_asset_spec(self):
        from pyramid_skins.configuration import register_path
        register_path(self.config, "pyramid_skins:tests/skins")

        from pyramid_skins.interfaces import ISkinObject
        inst = self.config.registry.queryUtility(ISkinObject, name="index.pt")
        self.assertTrue(inst is not None)

    def test_register_view(self):
        name = 'index.pt'
        from pyramid_skins.configuration import register_path
        register_path(self.config, os.path.join(path, 'skins'), indexes=[name])

        from pyramid.testing import DummyRequest
        from pyramid.view import render_view

        response = render_view('Hello world!', DummyRequest(), "")
        self.assertTrue(response is not None)



from pyramid_skins.models import BindableSkinObject, SkinObject  # noqa
from pyramid_skins.routes import RoutesTraverserFactory
from pyramid_skins.configuration import register_path


def includeme(config):
    """ Function meant to be included via
    :meth:`pyramid.config.Configurator.include`, which sets up the
    Configurator with a ``register_path`` method."""
    from pyramid_skins.interfaces import ISkinObjectFactory
    from pyramid_skins.models import SkinTemplate

    config.add_directive('register_path', register_path, action_wrap=False)
    config.add_traverser(RoutesTraverserFactory)
    config.registry.registerUtility(SkinTemplate, ISkinObjectFactory, name='.pt')



##############################################################################
#
# Copyright (c) 2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os
import sys

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt'), 'rb').read()
CHANGES = open(os.path.join(here, 'CHANGES.txt'), 'rb').read()

long_description = b"\n\n".join((README, CHANGES))
long_description = long_description.decode('utf-8')

requires = [
    'setuptools',
    'Chameleon >= 2.7',
    'zope.interface',
    'pyramid >= 1.3',
    ]

docs_extras = [
    'Sphinx',
    'docutils',
    'repoze.sphinx.autointerface']

testing_extras = [
    'manuel',
    'coverage',
    'nose',
    'pyramid_zcml >= 0.9.2',
    'zope.testing>3.8.7',
    'zope.component>3.9.2']

if sys.platform == "darwin":
    testing_extras.append("MacFSEvents")

if sys.platform.startswith("linux"):
    testing_extras.append("pyinotify")

setup(name='pyramid_skins',
      version = '2.0',
      description='Templating framework for Pyramid.',
      long_description=long_description,
      keywords = "pyramid templates",
      classifiers = [
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Zope Public License',
          'Programming Language :: Python',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          'Framework :: BFG'],
      license='BSD',
      author='Malthe Borch and the Pylons Community',
      author_email='pylons-discuss@googlegroups.org',
      maintainer='Malthe Borch',
      maintainer_email='mborch@gmail.com',
      url='http://packages.python.org/pyramid_skins/',
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages=[],
      include_package_data = True,
      zip_safe = False,
      install_requires=requires,
      tests_require=testing_extras,
      extras_require={
          'testing': testing_extras,
          'docs': docs_extras,
          'development': testing_extras + docs_extras},
      test_suite="pyramid_skins.tests",
      )



import os
import mimetypes
import functools

from zope.interface import implementer, provider
try:
    from zope.interface.interfaces import ComponentLookupError
except ImportError:
    from zope.component import ComponentLookupError

from chameleon.zpt.template import PageTemplateFile
from chameleon.tales import ProxyExpr
from chameleon.tales import TalesExpr

from pyramid.compat import string_types
from pyramid.response import Response
from pyramid.threadlocal import get_current_request
from pyramid.threadlocal import get_current_registry
from pyramid.url import route_url

from pyramid_skins.interfaces import ISkinObject
from pyramid_skins.interfaces import ISkinObjectFactory


def _lookup_component(request, name):
    registry = get_current_registry()
    return registry.queryAdapter(request, ISkinObject, name=name) or \
           registry.getUtility(ISkinObject, name=name)


def get_route_url_from_threadlocal_request(route_name):
    name = "/" + route_name.strip()
    request = get_current_request()
    return route_url(name, request)


def lookup_skin(template, name):
    request = get_current_request()
    name = name.strip()

    if name.startswith('/'):
        return _lookup_component(request, name[1:])

    assert ISkinObject.providedBy(template)

    path = '/' + template.name
    while path:
        try:
            return _lookup_component(request, "%s/%s" % (path[1:], name))
        except ComponentLookupError:
            pass

        path = path[:path.rindex('/')]

    return _lookup_component(request, name)


# BBB: Chameleon < 2.10.
if TalesExpr not in ProxyExpr.__bases__:
    _proxy_expr = ProxyExpr

    class ProxyExpr(TalesExpr):
        def __init__(self, name, *args):
            super(ProxyExpr, self).__init__(*args)
            self.name = name

        def translate(self, expression, target):
            """Return statements that assign a value to ``target``."""

            compiler = _proxy_expr(self.name, expression)
            return compiler(target, None)


@implementer(ISkinObject)
@provider(ISkinObjectFactory)
class SkinObject(object):

    _bound_kwargs = {}
    content_type = 'application/octet-stream'
    encoding = None

    def __init__(self, relative_path, path=None):
        self.path = path
        self.name = self.component_name(relative_path)

        if path is not None:
            content_type, encoding = mimetypes.guess_type(path)
            if content_type is not None:
                self.content_type = content_type
            if encoding is not None:  # pragma: nocover
                self.encoding = encoding

    def __call__(self, context=None, request=None, **kw):
        if request is None:
            request = get_current_request()
        if self.path is None:
            registry = request.registry
            inst = registry.queryAdapter(request, ISkinObject, name=self.name)
            if inst is None:
                inst = registry.getUtility(ISkinObject, name=self.name)
            return inst(context=context, request=request, **kw)

        result = self.render(context=context, request=request, **kw)
        if isinstance(result, string_types):
            response = Response(body=result)
        else:
            response = Response(app_iter=result)
            response.content_length = os.path.getsize(self.path)

        content_type = self.content_type
        if content_type is None:
            content_type = type(self).content_type
        response.content_type = content_type
        response.charset = self.encoding
        return response

    def __repr__(self):
        return '<%s.%s name="%s" path="%s" at 0x%x>' % (
            type(self).__module__, type(self).__name__,
            self.name, self.path, id(self))

    @classmethod
    def component_name(cls, relative_path):
        return relative_path

    def refresh(self):
        pass

    def render(self, **kw):
        return open(self.path, 'rb')


class BindableSkinObject(SkinObject):
    def __get__(self, view=None, cls=None):
        request = get_current_request()
        registry = get_current_registry()
        inst = registry.queryAdapter(request, ISkinObject, name=self.name)
        if inst is None:
            inst = registry.getUtility(ISkinObject, name=self.name)

        if view is None:
            return inst

        return functools.partial(inst.__call__, view=view, **view.__dict__)


class SkinTemplate(SkinObject, PageTemplateFile):
    content_type = 'text/html'
    encoding = 'UTF-8'

    expression_types = PageTemplateFile.expression_types.copy()
    expression_types['skin'] = functools.partial(ProxyExpr, '__skin')
    expression_types['route'] = functools.partial(ProxyExpr, '__route')

    @property
    def builtins(self):
        return {
            'macros': self.macros,
            'nothing': None,
            '__skin': functools.partial(lookup_skin, self),
            '__route': get_route_url_from_threadlocal_request,
            }

    def __init__(self, relative_path, path):
        SkinObject.__init__(self, relative_path, path)
        PageTemplateFile.__init__(self, path)

    def refresh(self):
        body = self.read()
        self.cook(body)

    def render(self, **context):
        return PageTemplateFile.render(self, **context)

    @classmethod
    def component_name(cls, relative_path):
        return os.path.splitext(relative_path)[0]

    def _set_path(self, path):
        self.filename = path

    def _get_path(self):
        return self.filename

    path = property(_get_path, _set_path)



import os
import unittest
from zope.interface import alsoProvides

path = os.path.dirname(__file__)


class RendererTest(unittest.TestCase):
    def setUp(self):
        import pyramid.testing
        self.config = pyramid.testing.setUp(
            hook_zca=True,
            request=pyramid.testing.DummyRequest(),
            autocommit=True
            )

        import pyramid_skins
        self.config.include(pyramid_skins)

    def tearDown(self):
        import pyramid.testing
        pyramid.testing.tearDown()

    def test_renderer(self):
        from pyramid_skins.configuration import register_path
        register_path(self.config, os.path.join(path, 'skins'))

        from pyramid_skins.renderer import renderer_factory
        self.config.add_renderer('skin', renderer_factory)

        def index(context, request):
            return {'context': context}

        self.config.add_view(index, name='index', renderer='skin')

        from pyramid.view import render_view
        from pyramid.threadlocal import get_current_request
        result = render_view('Hello world!', get_current_request(), 'index')
        self.assertTrue(b'Hello world' in result)

    def register_skins(self, paths):
        from pyramid.interfaces import IRequest
        from pyramid_skins.configuration import register_path

        skins = {}
        for path in paths:
            name = os.path.basename(path)
            class interface(IRequest):
                __doc__ = """ marker interface for custom theme """
            interface.__name__ = 'IThemeRequest-%s' % name
            skin = register_path(self.config, path, request_type=interface)
            skins[name] = dict(
                skin=skin,
                interface=interface)
        return skins

    def test_multiple_skins_alt(self):
        from pyramid.threadlocal import get_current_request
        from pyramid.view import render_view
        from pyramid_skins.configuration import register_path
        from pyramid_skins.renderer import renderer_factory

        register_path(self.config, os.path.join(path, 'skins'))
        skins = self.register_skins([
            os.path.join(path, 'alt_skins'),
            os.path.join(path, 'other_skins')])
        self.config.add_renderer('skin', renderer_factory)

        def index(context, request):
            return {'context': context}

        self.config.add_view(index, name='index', renderer='skin')

        alsoProvides(get_current_request(), skins['alt_skins']['interface'])
        result = render_view('Hello world!', get_current_request(), 'index')
        self.assertTrue(b'Alternative' in result)

    def test_multiple_skins_other(self):
        from pyramid.threadlocal import get_current_request
        from pyramid.view import render_view
        from pyramid_skins.configuration import register_path
        from pyramid_skins.renderer import renderer_factory

        register_path(self.config, os.path.join(path, 'skins'))
        skins = self.register_skins([
            os.path.join(path, 'alt_skins'),
            os.path.join(path, 'other_skins')])
        self.config.add_renderer('skin', renderer_factory)

        def index(context, request):
            return {'context': context}

        self.config.add_view(index, name='index', renderer='skin')

        alsoProvides(get_current_request(), skins['other_skins']['interface'])
        result = render_view('Hello world!', get_current_request(), 'index')
        self.assertTrue(b'Other' in result)

    def test_skin_reload(self):
        from pyramid_skins.configuration import register_path
        import shutil
        import tempfile

        register_path(self.config, os.path.join(path, 'skins'))
        tmp = tempfile.mkdtemp()

        try:
            skins_dir = os.path.join(tmp, 'skins')
            os.mkdir(skins_dir)
            with open(os.path.join(skins_dir, 'index.pt'), 'wb') as f:
                f.write(b'<html><title>Alternative</title></html>')

            skins = self.register_skins([skins_dir])

            from pyramid_skins.renderer import renderer_factory
            self.config.add_renderer('skin', renderer_factory)

            def index(context, request):
                return {'context': context}

            self.config.add_view(index, name='index', renderer='skin')

            from pyramid.view import render_view
            from pyramid.threadlocal import get_current_request
            alsoProvides(get_current_request(), skins['skins']['interface'])
            result = render_view('Hello world!', get_current_request(), 'index')
            self.assertTrue(b'Alternative' in result)
            with open(os.path.join(skins_dir, 'index.pt'), 'wb') as f:
                f.write(b'<html><title>Other</title></html>')
            skins['skins']['skin'].configure()
            result = render_view('Hello world!', get_current_request(), 'index')
            self.assertTrue(b'Other' in result)
        finally:
            shutil.rmtree(tmp)



from zope import interface
from pyramid.interfaces import IView


class ISkinObject(interface.Interface):
    name = interface.Attribute(
        """Component name.""")

    def refresh():
        """Refresh object from file on disk (reload)."""


class ISkinObjectFactory(interface.Interface):
    def component_name(relative_path):
        """Returns a component name. This method must be a
        class- or static method."""


class ISkinTemplate(interface.Interface):
    """Skin templates are page templates which reside in a skin
    directory. These are registered as named components adapting on
    (context, request)."""

    name = interface.Attribute(
        """This is the basename of the template filename relative to
        the skin directory. Note that the OS-level path separator
        character is replaced with a forward slash ('/').""")

    path = interface.Attribute(
        """Full path to the template. This attribute is available to
    allow applications to get a reference to files that relate to the
    template file, e.g. metadata or additional resources. An example
    could be a title and a description, or a icon that gives a visual
    representation for the template.""")

    def __call__(context, request):
        """Returns a bound skin template instance."""

    def get_api(name):
        """Look up skin api by name."""

    def get_macro(name):
        """Look up skin macro by name."""


class IBoundSkinTemplate(ISkinTemplate):
    """Bound to a context and request."""

    def __call__(**kwargs):
        """Renders template to a response object."""

    def render(**kwargs):
        """Renders template to a unicode string, passing optional
        keyword-arguments."""


class ISkinTemplateView(IView):
    """When skin templates are set to provide one or more interfaces,
    a component providing this interface will be registered."""

    template = interface.Attribute(
        """The skin template object.""")

    def __call__():
        """Renders template to a response object."""

    def render(**kwargs):
        """Renders template to a unicode string, passing optional
        keyword-arguments."""


class ISkinApi(interface.Interface):
    """A helper component available to skin templates. Skin APIs
    should be registered as named components adapting on (context,
    request, template)."""


class ISkinApiMethod(interface.Interface):
    """Skin API methods are an alternative to a generic API and get
    the chance of having arguments passed."""
